@@ -17,7 +17,7 @@
 #include "cli-cmd.h"
 #include "cli-mem-types.h"
 
-#include "gf-event.h"
+#include <glusterfs/gf-event.h>
 
 #include <fnmatch.h>
 
@@ -86,10 +86,10 @@ cli_rl_err(struct cli_state *state, const char *fmt, va_list ap)
 void
 cli_rl_process_line(char *line)
 {
-    struct cli_state *state = NULL;
+    cli_state_t *state = THIS->private;
     int ret = 0;
 
-    state = global_state;
+    GF_ASSERT(state);
 
     state->rl_processing = 1;
     {
@@ -104,15 +104,15 @@ cli_rl_process_line(char *line)
 
 void
 cli_rl_stdin(int fd, int idx, int gen, void *data, int poll_out, int poll_in,
-             int poll_err)
+             int poll_err, char event_thread_died)
 {
-    struct cli_state *state = NULL;
+    struct cli_state *state = data;
 
-    state = data;
+    GF_ASSERT(state);
 
     rl_callback_read_char();
 
-    event_handled(state->ctx->event_pool, fd, idx, gen);
+    gf_event_handled(state->ctx->event_pool, fd, idx, gen);
 
     return;
 }
@@ -120,10 +120,10 @@ cli_rl_stdin(int fd, int idx, int gen, void *data, int poll_out, int poll_in,
 char *
 cli_rl_autocomplete_entry(const char *text, int times)
 {
-    struct cli_state *state = NULL;
+    cli_state_t *state = THIS->private;
     char *retp = NULL;
 
-    state = global_state;
+    GF_ASSERT(state);
 
     if (!state->matchesp)
         return NULL;
@@ -281,11 +281,9 @@ cli_rl_autocomplete_prepare(struct cli_state *state, const char *text)
             break;
     }
 
-    if (!word)
+    if (!word || !token)
         goto out;
 
-    if (!token)
-        return 0;
     matches = cli_rl_get_matches(state, word, token);
 
     state->matches = matches;
@@ -311,11 +309,11 @@ cli_rl_autocomplete_cleanup(struct cli_state *state)
 char **
 cli_rl_autocomplete(const char *text, int start, int end)
 {
-    struct cli_state *state = NULL;
+    cli_state_t *state = THIS->private;
     char **matches = NULL;
     char save = 0;
 
-    state = global_state;
+    GF_ASSERT(state);
 
     /* hack to make the autocompletion code neater */
     /* fake it as though the cursor is at the end of line */
@@ -343,11 +341,14 @@ complete_none(const char *txt, int times)
 void *
 cli_rl_input(void *_data)
 {
-    struct cli_state *state = NULL;
+    cli_state_t *state = _data;
     char *line = NULL;
 
-    state = _data;
+    GF_ASSERT(state);
 
+    fprintf(stderr,
+            "Welcome to gluster prompt, type 'help' to see the available "
+            "commands.\n");
     for (;;) {
         line = readline(state->prompt);
         if (!line)
@@ -363,7 +364,7 @@ cli_rl_input(void *_data)
 }
 
 int
-cli_rl_enable(struct cli_state *state)
+cli_rl_enable(cli_state_t *state)
 {
     int ret = 0;
 
@@ -378,7 +379,8 @@ cli_rl_enable(struct cli_state *state)
         goto out;
     }
 
-    ret = event_register(state->ctx->event_pool, 0, cli_rl_stdin, state, 1, 0);
+    ret = gf_event_register(state->ctx->event_pool, 0, cli_rl_stdin, state, 1,
+                            0, 0);
     if (ret == -1)
         goto out;
 
@@ -392,7 +394,7 @@ out:
 #else /* HAVE_READLINE */
 
 int
-cli_rl_enable(struct cli_state *state)
+cli_rl_enable(cli_state_t *state)
 {
     return 0;
 }
