@@ -13,15 +13,23 @@
 #include <unistd.h>
 
 #include <glusterfs/glusterfs.h>
+#include <glusterfs/syscall.h>
 #include "glfs-internal.h"
 #include "rpc-clnt.h"
 #include "protocol-common.h"
 #include "xdr-generic.h"
 #include "glusterd1-xdr.h"
 
+/* In seconds */
+#define CONNECT_TIMEOUT 60
+#define REPLY_TIMEOUT 120
+
 int done = 0;
 int rpc_status;
 
+pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
+
 struct rpc_clnt_procedure gf_attach_actors[GLUSTERD_BRICK_MAXVALUE] = {
     [GLUSTERD_BRICK_NULL] = {"NULL", NULL},
     [GLUSTERD_BRICK_OP] = {"BRICK_OP", NULL},
@@ -38,8 +46,12 @@ struct rpc_clnt_program gf_attach_prog = {
 int32_t
 my_callback(struct rpc_req *req, struct iovec *iov, int count, void *frame)
 {
+    pthread_mutex_lock(&mutex);
     rpc_status = req->rpc_status;
     done = 1;
+    /* Signal main thread which is the only waiter */
+    pthread_cond_signal(&cond);
+    pthread_mutex_unlock(&mutex);
     return 0;
 }
 
@@ -48,6 +60,7 @@ int
 send_brick_req(xlator_t *this, struct rpc_clnt *rpc, char *path, int op)
 {
     int ret = -1;
+    struct timespec ts;
     struct iobuf *iobuf = NULL;
     struct iobref *iobref = NULL;
     struct iovec iov = {
@@ -57,7 +70,6 @@ send_brick_req(xlator_t *this, struct rpc_clnt *rpc, char *path, int op)
     call_frame_t *frame = NULL;
     gd1_mgmt_brick_op_req brick_req;
     void *req = &brick_req;
-    int i;
 
     brick_req.op = op;
     brick_req.name = path;
@@ -75,10 +87,6 @@ send_brick_req(xlator_t *this, struct rpc_clnt *rpc, char *path, int op)
     if (!iobref)
         goto out;
 
-    frame = create_frame(this, this->ctx->pool);
-    if (!frame)
-        goto out;
-
     iobref_add(iobref, iobuf);
 
     iov.iov_base = iobuf->ptr;
@@ -91,20 +99,44 @@ send_brick_req(xlator_t *this, struct rpc_clnt *rpc, char *path, int op)
 
     iov.iov_len = ret;
 
-    for (i = 0; i < 60; ++i) {
-        if (rpc->conn.connected) {
-            break;
-        }
-        sleep(1);
+    /* Wait for connection */
+    timespec_now_realtime(&ts);
+    ts.tv_sec += CONNECT_TIMEOUT;
+    pthread_mutex_lock(&rpc->conn.lock);
+    {
+        while (!rpc->conn.connected)
+            if (pthread_cond_timedwait(&rpc->conn.cond, &rpc->conn.lock, &ts) ==
+                ETIMEDOUT) {
+                fprintf(stderr, "timeout waiting for RPC connection\n");
+                pthread_mutex_unlock(&rpc->conn.lock);
+                return EXIT_FAILURE;
+            }
+    }
+    pthread_mutex_unlock(&rpc->conn.lock);
+
+    frame = create_frame(this, this->ctx->pool);
+    if (!frame) {
+        ret = -1;
+        goto out;
     }
 
     /* Send the msg */
     ret = rpc_clnt_submit(rpc, &gf_attach_prog, op, my_callback, &iov, 1, NULL,
                           0, iobref, frame, NULL, 0, NULL, 0, NULL);
     if (!ret) {
-        for (i = 0; !done && (i < 120); ++i) {
-            sleep(1);
+        /* OK, wait for callback */
+        timespec_now_realtime(&ts);
+        ts.tv_sec += REPLY_TIMEOUT;
+        pthread_mutex_lock(&mutex);
+        {
+            while (!done)
+                if (pthread_cond_timedwait(&cond, &mutex, &ts) == ETIMEDOUT) {
+                    fprintf(stderr, "timeout waiting for RPC reply\n");
+                    pthread_mutex_unlock(&mutex);
+                    return EXIT_FAILURE;
+                }
         }
+        pthread_mutex_unlock(&mutex);
     }
 
 out:
@@ -132,6 +164,54 @@ usage(char *prog)
     return EXIT_FAILURE;
 }
 
+static int
+sanitize_args(int argc, char **argv)
+{
+    int ret = 0;
+    struct stat statbuf = {
+        0,
+    };
+
+    ret = sys_lstat(argv[optind], &statbuf);
+    if (ret == -1) {
+        fprintf(stderr, "Unable to stat %s (%s).\n", argv[optind],
+                strerror(errno));
+        goto err;
+    }
+    if (!S_ISSOCK(statbuf.st_mode)) {
+        fprintf(stderr, "%s: Invalid socket file.\n", argv[optind]);
+        goto err;
+    }
+
+    ret = sys_lstat(argv[optind + 1], &statbuf);
+    if (ret == -1) {
+        fprintf(stderr, "Unable to stat %s (%s).\n", argv[optind + 1],
+                strerror(errno));
+        goto err;
+    }
+
+    switch (argc) {
+        case 3:
+            /* attach request */
+            if (!S_ISREG(statbuf.st_mode)) {
+                fprintf(stderr, "%s: Invalid volfile.\n", argv[optind + 1]);
+                goto err;
+            }
+            break;
+        case 4:
+            /* detach request */
+            if (!S_ISDIR(statbuf.st_mode)) {
+                fprintf(stderr, "%s: Invalid brick path.\n", argv[optind + 1]);
+                goto err;
+            }
+            break;
+    }
+
+    return 0;
+err:
+    return -1;
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -157,6 +237,11 @@ done_parsing:
         return usage(argv[0]);
     }
 
+    ret = sanitize_args(argc, argv);
+    if (ret != 0) {
+        return EXIT_FAILURE;
+    }
+
     fs = glfs_new("gf-attach");
     if (!fs) {
         fprintf(stderr, "glfs_new failed\n");
@@ -190,7 +275,7 @@ done_parsing:
         return EXIT_FAILURE;
     }
 
-    rpc = rpc_clnt_new(options, fs->ctx->master, "gf-attach-rpc", 0);
+    rpc = rpc_clnt_new(options, fs->ctx->root, "gf-attach-rpc", 0);
     if (!rpc) {
         fprintf(stderr, "rpc_clnt_new failed\n");
         return EXIT_FAILURE;
@@ -206,5 +291,5 @@ done_parsing:
         return EXIT_FAILURE;
     }
 
-    return send_brick_req(fs->ctx->master, rpc, argv[optind + 1], op);
+    return send_brick_req(fs->ctx->root, rpc, argv[optind + 1], op);
 }
