@@ -12,9 +12,9 @@
 #define _QUOTAD_AGGREGATOR_H
 
 #include "quota.h"
-#include "stack.h"
+#include <glusterfs/stack.h>
 #include "glusterfs3-xdr.h"
-#include "inode.h"
+#include <glusterfs/inode.h>
 
 typedef struct {
     void *pool;
@@ -23,13 +23,15 @@ typedef struct {
     inode_table_t *itable;
     loc_t loc;
     dict_t *xdata;
+    dict_t *req_xdata;
 } quotad_aggregator_state_t;
 
 typedef int (*quotad_aggregator_lookup_cbk_t)(xlator_t *this,
                                               call_frame_t *frame, void *rsp);
 int
-qd_nameless_lookup(xlator_t *this, call_frame_t *frame, gfs3_lookup_req *req,
-                   dict_t *xdata, quotad_aggregator_lookup_cbk_t lookup_cbk);
+qd_nameless_lookup(xlator_t *this, call_frame_t *frame, char *gfid,
+                   dict_t *xdata, char *volume_uuid,
+                   quotad_aggregator_lookup_cbk_t lookup_cbk);
 int
 quotad_aggregator_init(xlator_t *this);
 
